# encoding: utf-8

import django
from django.db import models

from . import queryset


class RandomManager(models.Manager):
    def random(self, *args, **kwargs):
        return self.get_queryset().random(*args, **kwargs)

    def get_queryset(self):
        return queryset.RandomQuerySet(self.model)



# encoding: utf-8

from django.db import models
from django.db.models import Count, Max, Min

from django_random_queryset import strategies


class RandomQuerySet(models.query.QuerySet):
    def random(self, amount=1):
        aggregates = self.aggregate(
            min_id=Min("id"), max_id=Max("id"), count=Count("id")
        )

        if not aggregates["count"]:
            return self.none()

        if aggregates["count"] <= amount:
            return self.all()

        if (aggregates["max_id"] - aggregates["min_id"]) + 1 == aggregates["count"]:
            return self.filter(
                id__in=strategies.min_max(
                    amount,
                    aggregates["min_id"],
                    aggregates["max_id"],
                    aggregates["count"],
                )
            )

        try:
            selected_ids = strategies.min_max_count(
                amount, aggregates["min_id"], aggregates["max_id"], aggregates["count"]
            )
        except strategies.SmallPopulationSize:
            selected_ids = self.values_list("id", flat=True)

        assert len(selected_ids) > amount
        return self.filter(id__in=selected_ids).order_by("?")[:amount]



from setuptools import setup

setup(
    name='django-random-queryset',
    version='0.1.3',
    author='Roman M. Remizov',
    author_email='rremizov@yandex.ru',
    license='MIT',
    platforms=['any'],
    description="Pull random records using Django ORM.",
    long_description=open('README.rst').read(),
    url='http://github.com/rremizov/django-random-queryset',
    packages=['django_random_queryset'],
    install_requires=['django>=1.11'],
    test_suite='tests.run_tests.run_all',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# encoding: utf-8
from __future__ import division

import math
import random


class SmallPopulationSize(Exception):
    pass


def min_max(sample_size, min_id, max_id, rows_count):
    assert (max_id - min_id) + 1 == rows_count
    population = list(range(min_id, max_id + 1))
    return random.sample(population, sample_size)


def min_max_count(desired_sample_size, min_id, max_id, rows_count):
    assert (max_id - min_id) + 1 >= rows_count, "{}, {}, {}".format(
        min_id, max_id, rows_count
    )
    id_holes_amount = (max_id - min_id) + 1 - rows_count
    id_holes_ratio = id_holes_amount / (max_id - min_id)
    assert 0 <= id_holes_ratio <= 1

    population = list(range(min_id, max_id + 1))
    actual_sample_size = int(
        math.ceil(desired_sample_size * max(1, math.ceil(1 / id_holes_ratio)))
    )
    actual_sample_size *= 10

    if actual_sample_size >= rows_count or actual_sample_size > len(population):
        raise SmallPopulationSize()

    return random.sample(population, actual_sample_size)
